import logging
import logging.config
import os

import yaml

fileLogger = logging.getLogger("fileLogger")


def init_logger(conf_file):
    if os.path.exists(conf_file):
        with open(conf_file, 'r') as f:
            config = yaml.safe_load(f.read())
        logging.config.dictConfig(config)


def debug_log(file_name, msg):
    if fileLogger.hasHandlers():
        if len(file_name.strip()) > 0:
            fileLogger.handlers[0].setFileName("debug_" + file_name)
        else:
            fileLogger.handlers[0].setFileName(fileLogger.handlers[0].defaultFileName)
        fileLogger.debug(msg)


def error_log(file_name, msg):
    if fileLogger.hasHandlers():
        if len(file_name.strip()) > 0:
            fileLogger.handlers[0].setFileName("error_" + file_name)
        else:
            fileLogger.handlers[0].setFileName(fileLogger.handlers[0].defaultFileName)
        if isinstance(msg, Exception):
            try:
                fileLogger.error(msg, exc_info=True)
            except Exception:
                fileLogger.error(str(msg))
        else:
            fileLogger.error(str(msg))



import logging
import multiprocessing
import os
import sys
import threading
import time
import traceback
from datetime import datetime
from logging.handlers import RotatingFileHandler


class MultiProcessingLog(logging.Handler):
    last_date = ''
    log_name = ''
    defaultFileName = ""
    dir_name = ""
    mode = ''
    maxBytes = ''
    backupCount = ''

    def __init__(self, name, mode, maxsize, rotate):
        logging.Handler.__init__(self)

        directory = "log/" + datetime.today().strftime('%Y-%m-%d')
        self.dir_name = directory
        self.log_name = name
        if not os.path.exists(directory):
            os.makedirs(directory, mode=0o777)

        filename = directory + "/" + name + "_" + datetime.today().strftime('%Y-%m-%d') + ".log"

        self.defaultFileName = name
        self._handler = RotatingFileHandler(
            filename=filename, mode=mode,
            maxBytes=maxsize, backupCount=rotate)
        self.mode = mode
        self.maxBytes = maxsize
        self.backupCount = rotate
        self.queue = multiprocessing.Queue(-1)
        t = threading.Thread(target=self.receive)
        t.daemon = True
        t.start()
        self.last_date = datetime.today().strftime('%Y-%m-%d')

    def setFormatter(self, fmt):
        logging.Handler.setFormatter(self, fmt)
        self._handler.setFormatter(fmt)

    def receive(self):
        while True:
            try:
                today = datetime.today().strftime('%Y-%m-%d')
                if self.last_date != today:
                    directory = "log/" + today
                    self.dir_name = directory
                    if not os.path.exists(directory):
                        os.makedirs(directory, mode=0o777)
                    filename = directory + "/" + self.log_name + "_" + today + ".log"
                    abs_path = os.path.abspath(filename)
                    if self._handler.baseFilename != abs_path:
                        self._handler.baseFilename = abs_path
                        temp_formatter = self._handler.formatter
                        self._handler = RotatingFileHandler(
                            filename=filename, mode=self.mode,
                            maxBytes=self.maxBytes, backupCount=self.backupCount)
                        self._handler.formatter = temp_formatter
                    self.last_date = today
                record = self.queue.get()
                self._handler.emit(record)
                # print('received on pid {}'.format(os.getpid()))
            except (KeyboardInterrupt, SystemExit):
                raise
            except EOFError:
                break
            except Exception as e:
                traceback.print_exc(file=sys.stderr)

    def send(self, s):
        self.queue.put_nowait(s)

    def _format_record(self, record):

        if record.args:
            record.msg = record.msg % record.args
            record.args = None
        if record.exc_info:
            self.format(record)
            record.exc_info = None

        return record

    def emit(self, record):
        try:
            s = self._format_record(record)
            self.send(s)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def close(self):
        while not self.queue.empty():
            time.sleep(.1)
        self._handler.close()
        logging.Handler.close(self)

    def setFileName(self, file_name):
        try:
            today = datetime.today().strftime('%Y-%m-%d')
            if self.last_date != today:
                directory = "log/" + today
                self.dir_name = directory
                if not os.path.exists(directory):
                    os.makedirs(directory, mode=0o777)

            filename = self.dir_name + "/" + file_name + "_" + datetime.today().strftime('%Y-%m-%d') + ".log"
            abs_path = os.path.abspath(filename)
            if self._handler.baseFilename != abs_path:
                self._handler.baseFilename = abs_path
                temp_formatter = self._handler.formatter
                self._handler = RotatingFileHandler(
                    filename=filename, mode=self.mode,
                    maxBytes=self.maxBytes, backupCount=self.backupCount)
                self._handler.formatter = temp_formatter
        except Exception as e:
            print(e)



class MException(Exception):
    """Custom JSON based exception.

    :param code: response code
    :param message: exception message
    """
    message = ''

    def __init__(self, code=None, message=None):
        Exception.__init__(self)
        if message is not None:
            self.message = message

        if code is not None:
            self.code = code

    def to_dict(self):
        return {
            'code': self.code,
            'desc': self.message
            # 'type': str(self.__class__.__name__)
        }


class InvalidRequestType(MException):
    """
    Raised when an invalid Request-Type is provided.
    """
    pass



import json
from functools import singledispatch

from flask import jsonify, Response

from .exception import MException


@singledispatch
def to_serializable(rv):
    """
    Define a generic serializable function.
    """
    pass


@to_serializable.register(float)
def ts_dict(rv):
    """Register the `float` type
    for the generic serializable function.
    :param rv: object to be serialized
    :type rv: float
    :returns: flask Response object
    """
    response = {
        'code': 0,
        'desc': "",
        'result': str(rv)
    }
    return jsonify(response)


@to_serializable.register(dict)
def ts_dict(rv):
    """Register the `dict` type
    for the generic serializable function.
    :param rv: object to be serialized
    :type rv: dict
    :returns: flask Response object
    """
    response = {
        'code': 0,
        'desc': "",
        'result': json.dumps(rv)
        # 'result': rv
    }
    return jsonify(response)


@to_serializable.register(list)
def ts_list(rv):
    """Register the `list` type
    for the generic serializable function.
    :param rv: objects to be serialized
    :type rv: list
    :returns: flask Response object
    """
    return Response(json.dumps(rv, indent=4, sort_keys=True))


class JSONResponse(Response):
    """
    Custom `Response` class that will be
    used as the default one for the application.
    All responses will be of type
    `application-json`.
    """

    @classmethod
    def force_type(cls, rv, environ=None):
        rv = to_serializable(rv)
        return super(JSONResponse, cls).force_type(rv, environ)


def json_error_handler(app):
    @app.errorhandler(MException)
    def handle_invalid_usage(error):
        """
        Custom `Exception` class that will be
        used as the default one for the application.
        Returns pretty formatted JSON error
        with detailed information.

        :message: error message
        :code: response status code
        :type: error type
        """

        response = jsonify(error.to_dict())
        response.code = error.code
        return response

    @app.errorhandler(Exception)
    def handle_exception(error):
        """
        Handle unknown Exception
        """

        return jsonify({
            'code': 1,
            'desc': 'Алдаа гарлаа. Дахин оролдоно уу.'
        })



import compileall
import os
import shutil
from datetime import datetime
from pathlib import Path
from shutil import copyfile


def update(just_pyc=False):
    for subdir, dirs, files in os.walk('.'):
        if 'venv' in subdir or '.env' in subdir:
            continue
        if '__pycache__' in subdir:
            shutil.rmtree(subdir)

    compileall.compile_dir('.')
    if os.path.exists('update'):
        shutil.rmtree('update')

    Path('update').mkdir(parents=True, exist_ok=True)

    for subdir, dirs, files in os.walk('.'):
        if 'git' in subdir or 'update' in subdir or '.idea' in subdir or subdir.startswith(
                '.\\log') or 'venv' in subdir or 'test' in subdir:
            continue

        if '__pycache__' in subdir:
            for file in files:
                if 'update' in file:
                    continue
                filename = os.path.join('update', subdir.replace('__pycache__', ''), file)
                filename = filename.replace('.cpython-36', '')
                filename = filename.replace('.cpython-38', '')
                print('copying {}...'.format(os.path.join(subdir, file), filename))
                copyfile(os.path.join(subdir, file), filename)
            continue

        Path(os.path.join('update'), subdir).mkdir(parents=True, exist_ok=True)
        if just_pyc:
            continue

        for file in files:
            if 'git' in file or file.endswith('.py') or file.endswith('.zip') or file.endswith('.md') or file.endswith(
                    '.jpg'):
                continue
            subdir = subdir.replace('.', '') if len(subdir) == 1 else subdir
            filename = os.path.join('update', subdir) + '/' + file
            print('copying {}...'.format(os.path.join(subdir, file), filename))
            copyfile(os.path.join(subdir, file), filename)

    now = datetime.now()
    output_filename = 'update' + now.strftime('%Y%m%d-%H%M')
    shutil.make_archive(output_filename, 'zip', 'update')
    shutil.rmtree('update')



from distutils.core import setup

setup(
    name='gcm_common',
    packages=['gcm_common'],
    version='0.1.13',
    license='MIT',
    description='GCM common package',
    author='Batkhishig Dulamsurankhor',
    author_email='batkhishign55@gmail.com',
    keywords=['gcm', 'common', 'flask', 'python'],
    install_requires=[
        'flask',
        'PyYAML',
    ]
)
